// ======================================================================================
// 
//
// Function : Client Framework 
// Author   : Robin Mitchell
// Date		: 13/10/2016
// Version	: 0.0.1
// 
// Designed for the ESP8266 module using SDK 1.5.4 and AT 1.1.0.0
// Later versions may not work
// Earlier versions may have AT+CLOSE busy bugs!
// ======================================================================================

#ifndef __IOT_CLIENTFRAMEWORK
#define __IOT_CLIENTFRAMEWORK


// ======================================================================================
// UART IO
// ======================================================================================
	
	void uartSend(unsigned char byteToSend);
	void uartGet(unsigned char byteReceived);
	void uartInit(void);
	void delay20ms(void);


// ======================================================================================
// User Functions
// ======================================================================================

	// These are YOUR commands so do as you will with them!
		
	
	void internet_update(void);
	
	unsigned char internet_init(void);
	unsigned char internet_waitUntilDone(void);
	unsigned char internet_connectToWifi(rom const char *wifiName, rom const char *wifiPassword);
	unsigned char internet_disconnectFromWifi(void);
	unsigned char internet_sendData(char *data, char length);
	unsigned char internet_newDataAvailable(void);
	unsigned char internet_getDataLength(void);
	unsigned char internet_commandSuccess(void);
	unsigned char internet_connectionStatus(void);
	unsigned char internet_wifiStatus(void);
	unsigned char internet_setdeviceID(unsigned char id);


	unsigned char internet_connectToRIOTServer(rom const char *ip, rom const char *port);
	unsigned char internet_sendDataByID(char *data, char length, char id);

	char *internet_getNewData(void);
	

	
// ======================================================================================
// 			      <<<<<<<<<<<<<<<< FRAMEWORK INTERNALS >>>>>>>>>>>>>>>>>
// 
// 					DONT ACCESS OR EVEN TOUCH ANYTHING BELOW THIS LINE
// ======================================================================================


	// **********************************************************************************
	// Defines and Constants
	// **********************************************************************************	

		#define TRUE  1
		#define FALSE 0
		
		#define CR 0x0D
		#define LF 0x0A
		
		#define OK 			0
		#define SEND_OK 	1
		#define SEND_FAIL 	2
		#define SEND_ERROR	3
		#define ERROR		4
		#define FAIL		5
		#define BUSY_P		6
		#define BUSY_S		7
		#define IPD			8
		#define PROMPT		9
		#define CONNECT		10
		#define CLOSED		11
		#define WIFICONNECT	12
		#define WIFICLOSED	14	
			
		//		^ Equal to number of that ^
		#define NO_INSTRUCTIONS 13

		#define SEMICOLON	0x3A
		#define PROMPTSYM	0x3E
		#define COMMA		0x2C
		#define SPACE		0x20
		#define SPEECH		0x22
		
		#define RIOTDEVID	0x00
		#define RIOTDATA	0x01
		#define RIOTBYREF	0x02
		#define RIOTTIMEOUT	0x03
		


	// **********************************************************************************
	// Function Prototypes
	// **********************************************************************************
		unsigned char commandSend(rom const char *string);
		unsigned char commandValidate(void);
		
		void tcpHandle(void);
		void commandHandle(void);
		void dataHandle(void);
		void timeoutChecker(void);
		void cleanup(void);

	// **********************************************************************************
	
		
// ======================================================================================



#endif
