#include <p18f25k20.h>

#include "IoT_ClientFramework.h"
#include "setup.h"


// ======================================================================================
// ISR CODE FOR ESP8266 UART CONTROLS

	#pragma code isr=0x08
	#pragma interrupt isr
	
	void isr(void)
	{
		if(PIR1bits.RCIF)
		{
			PIR1bits.RCIF = 0;
			uartGet(RCREG);
			internet_update();	
		}	
		INTCONbits.PEIE = 1;		// Enable peripheral interrupts
	}
	
	#pragma code

// ======================================================================================


float time;
float time2;
unsigned char dataLen;
char dataBuff[1];




void main()
{	
	// Run the setup function found in setup.h
	setup();

	// Analog pin RA0 needs to be configured as an input
	TRISA = 0x01;
	
	// Keep LEDs off for now
	LATAbits.LATA1 = 0;
	LATAbits.LATA2 = 0;	
	
	// Initialise the ESP8266
	internet_init();
	
	// Connect to the network
	// This will keep looping until a network has been joined
	while(!internet_connectToWifi("SSID", "PASS"));

	// Turn on the WiFi Indicator LED
	LATAbits.LATA2 = 1;		
	
	// Connect to a RIOT server
	// This will connect to the advertised port and then
	// be given a dynamically allocated port number to 
	// do all communication on. The system will then 
	// connect to that port instead
	while(!internet_connectToRIOTServer("xxx.xxx.xxx.xxx","333"));
	
	// Turn on the Server connection LED
	LATAbits.LATA1 = 1;
	
	// Set the ID on the RIOT server
	// The ID is used for dynamic communication between clients
	// Instead of needing to know the IP address, an ID can be used instead (send to ID)
	while(!internet_setdeviceID(0));
	

	do
	{
		// A simple delay loop for making measurements occur every so often (a few a second)
		
		for(time = 0; time < 400; time++)
		{
		}
		
		
		// Star the ADC Conversion		
		ADCON0bits.GO = 1;
		
		
		// Wait until the ADC Conversion is complete
		while(ADCON0bits.GO);
		
		
		// Get the recorded byte
		dataBuff[0] = ADRESH;
		
		
		// Send the byte to the RIOT server. You MUST use this function for sending data to RIOT servers!
		// RIOT servers use a simple message protocol that consists of one letter at the beginning of 
		// a data stream that tells it what to do. For example, I is for ID and D is for data. This function
		// adds that single char message for you automatically, you only worry about your data
		riot_sendData(dataBuff, 1);
		
	}
	while(1);
}

