﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Net.Sockets


Public Class Server

    ' Public Constants that you may need to change '
    Public Const MAX_CONNECTION = 32
    Public Const BASEPORT = 60000
    Public Const BUFFERSIZE = 64

    ' Public Variables '
    Public clientBuffers(MAX_CONNECTION, BUFFERSIZE) As Byte
    Public clientLengths(MAX_CONNECTION) As Integer
    Public clientDataAvailable(MAX_CONNECTION) As Boolean

    Public Sub startServer()

        init()
        Dim mainServerThread As New System.Threading.Thread(AddressOf advertiseThread)
        Dim clientHandle As New System.Threading.Thread(AddressOf dataHandle)
        mainServerThread.Start()
        clientHandle.Start()

    End Sub

    ' By Port User Functions '
    Public Function getClientDataByPort(ByVal port As Integer, ByRef buffer As Byte(), ByRef dataLength As Integer) As Boolean
        If (clientDataAvailable(port) = False) Then
            Return False
        Else
            clientDataAvailable(port) = False
            dataLength = clientLengths(port)
            For i As Integer = 0 To BUFFERSIZE
                buffer(i) = clientBuffers(port, i)
            Next
            Return True
        End If
    End Function

    Public Function isClientConnected(ByVal id As Integer) As Boolean
        Return clients(id).inUse
    End Function

    Public Function sendDataToClient(ByVal id As Integer, ByRef buffer As Byte(), ByVal length As Byte)
        If (clients(id).inUse = True) Then
            Dim tempBuffer(buffer.Length + 1) As Byte
            tempBuffer(0) = 68
            For i As Integer = 0 To buffer.Length - 1
                tempBuffer(i + 1) = buffer(i)
            Next
            clients(id).sendData(tempBuffer, length + 1)
            Return True
        End If

        Return False
    End Function

    ' By Unique ID User Functions '
    Public Function getClientDataByID(ByVal ID As Integer, ByRef buffer As Byte(), ByRef dataLength As Integer) As Boolean
        ' Look for the unique ID in all the clients'
        For i As Integer = 0 To MAX_CONNECTION - 1
            If (clients(i).id = ID) Then
                If (clientDataAvailable(i) = False) Then
                    Return False
                End If
                clientDataAvailable(i) = False
                dataLength = clientLengths(i)
                For j As Integer = 0 To BUFFERSIZE
                    buffer(j) = clientBuffers(i, j)
                Next j
                Return True
            End If
        Next i
        Return False
    End Function

    Public Function isClientConnectedByID(ByVal id As Integer) As Boolean
        For i As Integer = 0 To MAX_CONNECTION - 1
            If (clients(i).id = id) Then
                Return clients(i).inUse
            End If
        Next
        Return 0
    End Function

    Public Function sendDataToClientByID(ByVal ID As Integer, ByRef buffer As Byte(), ByVal length As Byte)

        For i As Integer = 0 To MAX_CONNECTION - 1

            If (clients(i).id = ID) Then
                If (clients(i).inUse = True) Then
                    Dim tempBuffer(buffer.Length + 1) As Byte
                    tempBuffer(0) = 68
                    For j As Integer = 0 To buffer.Length - 1
                        tempBuffer(j + 1) = buffer(j)
                    Next
                    clients(i).sendData(tempBuffer, length + 1)
                    Return True
                End If
            End If
        Next
        Return False

    End Function


    ' -----------------------------------------------------------------'
    ' SERVER FRAMEWORK HERE '
    ' -----------------------------------------------------------------'
    Private advertisePort As Int32 = 333
    Private serverAddress As IPAddress = IPAddress.Parse("xxx.xxx.xxx.xxx")
    Private advertiseListen As TcpListener
    Private clients(MAX_CONNECTION) As client
    Private tempBuffer(BUFFERSIZE) As Byte
    Private dataSize As Integer


    Private Sub init()

        For counter As Integer = 0 To (MAX_CONNECTION - 1)
            clients(counter) = New client
        Next

    End Sub

    ''' <summary>
    ''' The server advertises on port 333 to assign ports
    ''' </summary>
    Private Sub advertiseThread()

        Dim server As TcpListener = New TcpListener(serverAddress, advertisePort)
        Dim client As TcpClient

        server.Start()

        While True
            client = server.AcceptTcpClient()
            Dim stream As NetworkStream = client.GetStream
            Dim clientPort As Integer = assignPort()
            Dim clientID As Integer = clientPort - BASEPORT
            Dim serverResponse As Byte() = System.Text.Encoding.ASCII.GetBytes(clientPort)
            stream.Write(serverResponse, 0, serverResponse.Length)
            clientPort = clientPort
            clients(clientID).start(clientPort, serverAddress)
            client.Close()
        End While
    End Sub

    ''' <summary>
    ''' Goes through all clients to find an empty port
    ''' </summary>
    Private Function assignPort() As Integer

        For baseCounter As Integer = 0 To (MAX_CONNECTION - 1)
            If (clients(baseCounter).inUse = False) Then
                clients(baseCounter).inUse = True
                Return BASEPORT + baseCounter
            End If
        Next

        Return BASEPORT + MAX_CONNECTION

    End Function

    ''' <summary>
    ''' This parses all data from clients 
    ''' </summary>
    Private Sub dataHandle()
        While True
            For i As Integer = 0 To MAX_CONNECTION - 1
                If (clients(i).inUse = True) Then
                    If (clients(i).newDataAvailable = True) Then
                        clients(i).getData(tempBuffer, dataSize)
                        clientLengths(i) = dataSize - 1
                        clientDataAvailable(i) = True

                        ' Parse the data for correct operation '

                        ' Is the data packet data for the server (D)? '
                        If (tempBuffer(0) = 68) Then
                            For j As Integer = 0 To BUFFERSIZE - 1
                                clientBuffers(i, j) = tempBuffer(j + 1)
                            Next j
                            ' Is the data packet the device ID (I)? '
                        ElseIf (tempBuffer(0) = 73) Then
                            clients(i).id = tempBuffer(1)
                            clientDataAvailable(i) = False
                            ' Is the data packet to be routed to another device (R)? '
                        ElseIf (tempBuffer(0) = 82) Then
                            Dim reroute As Integer = tempBuffer(1)
                            Dim tBuff(dataSize - 2) As Byte
                            For x As Integer = 0 To dataSize - 3
                                tBuff(x) = tempBuffer(x + 2)
                            Next x
                            ' Find the destination via ID '
                            For x As Integer = 0 To MAX_CONNECTION - 1
                                If (clients(x).id = reroute) Then
                                    clients(x).sendData(tBuff, tBuff.Length)
                                End If
                            Next x
                            clientDataAvailable(i) = False
                        Else
                            clientDataAvailable(i) = False
                        End If
                    End If
                End If
            Next i
        End While
    End Sub

End Class
