﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Net.Sockets
Imports System.Threading


Public Class client

    ' This class handles a single client '

    Public inUse As Boolean = False
    Public id As Integer
    Public newDataAvailable As Boolean = False

    Public Sub start(ByVal port As Int32, ByVal ip As IPAddress)
        Dim clThrd As New System.Threading.Thread(AddressOf clientThread)
        clientAddress = ip
        clientPort = port
        clThrd.Start()
    End Sub

    Public Sub getData(ByRef buffer As Byte(), ByRef dataLength As Integer)

        For i As Integer = 0 To bufferSize
            buffer(i) = clientBuffer(i)
        Next i
        dataLength = bufferSize
        newDataAvailable = False

    End Sub

    Public Sub sendData(ByRef buffer As Byte(), ByVal datalength As Integer)
        stream.Write(buffer, 0, datalength)
    End Sub

    ' =================================================================================== '
    ' CLIENT FRAMEWORK CODE '
    ' =================================================================================== '

    ' Constants '
    Private Const WATCHDOGTIMEOUT = 2000
    Private Const BUFFSIZE = 64

    ' Network objects '
    Private clientAddress As IPAddress
    Private clientListen As TcpListener
    Private client As TcpClient
    Private stream As NetworkStream

    ' Variables '
    Private clientBuffer(BUFFSIZE) As Byte
    Private bufferSize As Integer
    Private clientPort As Int32
    Private clientResponse() As Byte
    Private clientConnection As Boolean
    Private watchDogTimer As Int64



    ''' <summary>
    ''' This is the main thread for individual clients
    ''' </summary>
    Private Sub clientThread()

        clientListen = New TcpListener(clientAddress, clientPort)
        clientListen.Start()
        client = clientListen.AcceptTcpClient()
        clientConnection = True
        watchDogTimer = 0
        stream = client.GetStream

        ' Main connection loop'
        ' 1) Check WatchDog
        ' 2) If watchdog timeout reached probe for alive connection
        ' 3) Check for new data
        ' 4) If new data, reset watchdog

        While clientConnection = True

            If (stream.DataAvailable) Then
                bufferSize = stream.Read(clientBuffer, 0, clientBuffer.Length)
                newDataAvailable = True
                clientConnection = True
                watchDogTimer = 0
            End If

            If (watchDogTimer > WATCHDOGTIMEOUT) Then
                timeoutHandle()
            End If

            watchDogTimer = watchDogTimer + 1
            Thread.Sleep(10)

        End While

        cleanup()

    End Sub

    ''' <summary>
    ''' Handles the heartbeat
    ''' </summary>
    Private Sub timeoutHandle()

        Dim clientResponse As Byte() = System.Text.Encoding.ASCII.GetBytes("??")

        stream.Write(clientResponse, 0, clientResponse.Length)

        clientConnection = False
        Dim timeCounter As Integer = 0
        While (timeCounter < 600)
            If (stream.DataAvailable = True) Then
                clientConnection = True
                stream.Read(clientBuffer, 0, clientBuffer.Length)
                Exit While
            End If
            Thread.Sleep(10)
            timeCounter = timeCounter + 1
        End While

        If (clientConnection = False) Then
            clientResponse = System.Text.Encoding.ASCII.GetBytes("C")
            stream.Write(clientResponse, 0, clientResponse.Length)
            client.Close()
        Else
            clientResponse = System.Text.Encoding.ASCII.GetBytes("O")
            stream.Write(clientResponse, 0, clientResponse.Length)
            watchDogTimer = 0
        End If
    End Sub

    ''' <summary>
    ''' Cleans the client, closes the connection and prepares for new connection
    ''' </summary>
    Private Sub cleanup()
        clientListen.Stop()
        inUse = False
    End Sub

End Class
